import {
	type Editor,
	type MarkdownView,
	type MarkdownFileInfo,
	Notice,
	TFile,
} from "obsidian";
import { EditorCheckCommand } from "./Command";
import { RatingModal } from "@ui/modals";
import { STATUS_READ } from "@src/const";
import { mkdirRecursive, dirname } from "@utils/fs";
import moment from "@external/moment";
import type BookTrackerPlugin from "@src/main";

export class LogReadingFinishedCommand extends EditorCheckCommand {
	constructor(private readonly plugin: BookTrackerPlugin) {
		super("log-reading-finished", "Log Reading Finished");
	}

	private getPageCount(file: TFile): number {
		return this.plugin.getBookMetadata(file)?.pageCount ?? 0;
	}

	protected check(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): boolean {
		return !(
			ctx.file === null ||
			this.plugin.settings.statusProperty === "" ||
			this.plugin.settings.endDateProperty === "" ||
			this.plugin.settings.ratingProperty === "" ||
			this.plugin.settings.pageCountProperty === "" ||
			this.getPageCount(ctx.file) <= 0
		);
	}

	protected async run(
		_editor: Editor,
		ctx: MarkdownView | MarkdownFileInfo
	): Promise<void> {
		const file = ctx.file!;
		const fileName = file.basename;
		const pageCount = this.getPageCount(file);

		const ratings = await RatingModal.createAndOpen(
			this.plugin.app,
			this.plugin.settings.spiceProperty !== ""
		);

		try {
			await this.plugin.readingLog.createEntry(
				fileName,
				pageCount,
				pageCount
			);
		} catch (error) {
			new Notice(
				`Failed to log reading progress for ${fileName}: ${error}`
			);
			return;
		}

		const endDate = moment().format("YYYY-MM-DD");

		this.plugin.app.fileManager.processFrontMatter(file, (fm) => {
			fm[this.plugin.settings.statusProperty] = STATUS_READ;
			fm[this.plugin.settings.endDateProperty] = endDate;
			fm[this.plugin.settings.ratingProperty] = ratings.rating;
			if (this.plugin.settings.spiceProperty !== "") {
				fm[this.plugin.settings.spiceProperty] = ratings.spice;
			}
		});

		if (this.plugin.settings.organizeReadBooks) {
			const datePath = moment().format("YYYY/MMMM");
			const newPath = `${this.plugin.settings.readBooksFolder}/${datePath}/${file.name}`;

			await mkdirRecursive(this.plugin.app.vault, dirname(newPath));
			await this.plugin.app.vault.rename(file, newPath);
		}

		new Notice("Reading finished for " + fileName);
	}
}
